package com.persagy.fm.common.jms;

import com.google.gson.Gson;
import com.persagy.fm.common.constant.DefaultValueConstants;
import com.persagy.fm.common.context.AppContext;
import com.persagy.fm.common.jms.model.BaseMsg;
import com.persagy.fm.mybatis.handler.DynamicDataSourceHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 消息处理基类
 *
 * @author lixing
 * @version V1.0 2021/3/18 5:42 下午
 **/
@Service
public class MsgBaseHandler {
    @Autowired
    DynamicDataSourceHandler dynamicDataSourceHandler;

    /**
     * 重置数据源
     *
     * @param groupCode 集团编码
     * @author lixing
     * @version V1.0 2021/3/30 5:22 下午
     */
    private void resetDataSource(String groupCode) {
        AppContext.getContext().setGroupCode(groupCode);
        AppContext.getContext().setAppId(DefaultValueConstants.DEFAULT_APP_ID);
        dynamicDataSourceHandler.resetDataSource();
    }

    /**
     * 根据消息重置数据源
     *
     * @param message rabbitMQ消息
     * @author lixing
     * @version V1.0 2021/3/30 5:47 下午
     */
    public void resetDataSourceByMsg(String message) {
        BaseMsg baseMsg = getBaseMsg(message);
        resetDataSource(baseMsg.getGroupCode());
    }

    /**
     * 获取消息类型
     *
     * @param message rabbitMQ消息
     * @return 消息类型
     * @author lixing
     * @version V1.0 2021/3/18 6:16 下午
     */
    public String getMsgType(String message) {
        BaseMsg baseMsg = getBaseMsg(message);
        return baseMsg.getType();
    }

    /**
     * message转换为BaseMsg对象
     *
     * @param message rabbitMQ消息
     * @return BaseMsg对象
     * @author lixing
     * @version V1.0 2021/3/30 5:46 下午
     */
    private BaseMsg getBaseMsg(String message) {
        Gson gson = new Gson();
        return gson.fromJson(message, BaseMsg.class);
    }

}
