package com.persagy.fm.department.model.dto;

import com.persagy.fm.common.constant.RegexConstants;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

import lombok.Data;
import org.hibernate.validator.constraints.Length;

import java.util.List;

/**
 * @author lixing
 * @version V1.0 2021-03-11 15:09:18
 */
@Data
@ApiModel(value = "更新部门入参")
public class UpdateDepartmentDTO {
    @ApiModelProperty(value = "id", required = true)
    @NotBlank(message = "id不能为空")
    private String id;

    @ApiModelProperty(value = "部门名称", required = true)
    @Length(max = 100, message = "部门名称长度不能超过100字")
    @Pattern(regexp= RegexConstants.NAME_REGEX,
            message = "部门名称" + RegexConstants.NAME_REGEX_WARNING)
    private String name;

    @ApiModelProperty(value = "部门类型", notes = "com.persagy.fm.department.constant.enums.DepTypeEnum")
    private String type;

    @ApiModelProperty(value = "上级部门id")
    private String parentId;

    @ApiModelProperty(value = "数据来源", notes = "com.persagy.fm.department.constant.enums.ResourceFromEnum")
    private String resourceFrom;

    @ApiModelProperty(value = "负责人")
    private List<String> managers;

    @ApiModelProperty(value = "部门全路径")
    private String fullPath;

    @ApiModelProperty(value = "编制人数", hidden = true)
    private Integer staffCount;

    @ApiModelProperty(value = "关联的项目")
    private List<String> projectIds;
}
