package com.persagy.fm.department.service;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.fm.common.model.vo.SimpleObjVO;
import com.persagy.fm.department.model.dto.AddDepartmentDTO;
import com.persagy.fm.department.model.Department;
import com.persagy.fm.department.model.dto.PageQueryDepartmentDTO;
import com.persagy.fm.department.model.dto.QueryDepartmentDTO;
import com.persagy.fm.department.model.dto.UpdateDepartmentDTO;
import com.persagy.fm.department.model.vo.DepSimpleObjVO;
import com.persagy.fm.department.model.vo.FullPathInfoVO;

/**
 * 部门(Department) service接口
 *
 * @author lixing
 * @version V1.0 2021-03-11 15:09:17
 */
public interface IDepartmentService {

    /**
     * 创建部门
     *
     * @param addDepartmentDTO 创建部门入参
     * @return 部门主键
     * @author lixing
     * @version V1.0 2021-03-11 15:09:17
     */
    String createDepartment(AddDepartmentDTO addDepartmentDTO);

    /**
     * 创建部门
     *
     * @param department 部门对象
     * @return 部门主键
     * @author lixing
     * @version V1.0 2021/3/30 5:51 下午
     */
    String createDepartment(Department department);

    /**
     * 部门详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-03-11 15:09:17
     */
    Department queryDepartmentDetail(String id);

    /**
     * 更新部门
     *
     * @param updateDepartmentDTO 更新部门入参
     * @author lixing
     * @version V1.0 2021-03-11 15:09:17
     */
    void updateDepartment(UpdateDepartmentDTO updateDepartmentDTO);

    /**
     * 更新部门
     *
     * @param department 部门实体
     * @author lixing
     * @version V1.0 2021/3/30 3:50 下午
     */
    void updateDepartment(Department department);

    /**
     * 删除部门
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-03-11 15:09:17
     */
    void deleteDepartment(String id);

    /**
     * 查询部门
     *
     * @param queryDepartmentDTO 查询部门入参
     * @return List<Department>
     * @author lixing
     * @version V1.0 2021-03-11 15:09:17
     */
    List<Department> queryDepartmentList(QueryDepartmentDTO queryDepartmentDTO);

    /**
     * 分页查询部门
     *
     * @param pageQueryDepartmentDTO 分页查询入参
     * @return IPage<Department>
     * @author lixing
     * @version V1.0 2021-03-11 15:09:17
     */
    IPage<Department> pageQueryDepartment(PageQueryDepartmentDTO pageQueryDepartmentDTO);

    /**
     * 查询一组父部门的所有子部门
     *
     * @param parentDeptList 父部门列表
     * @return 子部门列表
     * @author lixing
     * @version V1.0 2021/3/12 4:22 下午
     */
    List<Department> getSonDeps(List<Department> parentDeptList);

    /**
     * 根据父部门id列表查询这些父部门的所有子部门
     *
     * @param parentIds 父部门id列表
     * @return 子部门列表
     * @author lixing
     * @version V1.0 2021/3/12 4:23 下午
     */
    List<Department> queryDepListByParentIds(List<String> parentIds);

    /**
     * 校验部门是否可删除
     *
     * @param id 部门主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021/3/12 11:02 上午
     */
    Department checkDeletable(String id);

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param department 部门实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    void setDefaultValue(Department department);

    /**
     * 获取部门信息
     *
     * @param depIds 部门id列表
     * @return 部门信息列表
     * @author lixing
     * @version V1.0 2021/4/1 6:06 下午
     */
    List<DepSimpleObjVO> getDepInfoList(List<String> depIds);

    /**
     * 获取当前登录人所在的集团部门
     *
     * @return 部门信息
     * @author lixing
     * @version V1.0 2021/4/25 10:54 上午
     */
    Department getGroupDep();

    /**
     * 获取部门的全路径信息
     *
     * @param depId 部门id
     * @return 部门全路径中包含的部门信息列表
     * @author lixing
     * @version V1.0 2021/4/26 4:59 下午
     */
    FullPathInfoVO getFullPathInfo(String depId);

    /**
     * 根据部门全路径查询子公司
     *
     * @param fullPath 部门全路径
     * @return 子公司列表
     * @author lixing
     * @version V1.0 2021/4/29 4:01 下午
     */
    List<Department> querySonDepsByFullPath(String fullPath);

    /**
     * 查询全路径中包含给定路径的公司
     *
     * @param fullPath 部门全路径
     * @return 公司列表
     * @author lixing
     * @version V1.0 2021/4/29 4:01 下午
     */
    List<Department> queryDepsContainsFullPath(String fullPath);

    /**
     * 根据全路径查询部门详情
     *
     * @param fullPath 部门全路径
     * @return 部门实体
     * @author lixing
     * @version V1.0 2021/4/30 5:35 下午
     */
    Department queryDepDetailByFullPath(String fullPath);
}
