package com.persagy.fm.department.utils;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.persagy.fm.department.model.Department;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 部门数据工具类
 *
 * @author lixing
 * @version V1.0 2021/3/12 3:06 下午
 **/
public class DepDataUtil {

    /**
     * 子部门列表转换为：子部门parentId -> 子部门列表的map
     *
     * @param sonDeps 子部门列表
     * @return map 子部门parentId -> 子部门列表
     * @author lixing
     * @version V1.0 2021/3/12 2:49 下午
     */
    public static Map<String, List<Department>> sonDeps2ParentIdMap(List<Department> sonDeps) {
        if (CollectionUtils.isEmpty(sonDeps)) {
            return Maps.newHashMap();
        }

        Map<String, List<Department>> result = new HashMap<>(sonDeps.size());
        sonDeps.forEach(sonDep -> {
            List<Department> departments = result.get(sonDep.getParentId());
            if (departments == null) {
                departments = Lists.newArrayList(sonDep);
                result.put(sonDep.getParentId(), departments);
            } else {
                departments.add(sonDep);
            }
        });

        return result;
    }


    /**
     * 判断一个部门是否为一组部门中某些部门的父部门
     *
     * @param depId 需要判断的父部门id
     * @param parentIdMap 子部门parentId -> 子部门列表
     * @return boolean
     * @author lixing
     * @version V1.0 2021/3/12 3:21 下午
     */
    public static boolean hasChildren(String depId, Map<String, List<Department>> parentIdMap) {
        if (depId == null) {
            return false;
        }
        return parentIdMap.get(depId) != null;
    }
}
