package com.persagy.fm.depproject.service;

import com.persagy.fm.depproject.model.*;
import com.persagy.fm.depproject.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.fm.saas.project.model.vo.ProjectTreeItemVO;

/**
 * 部门项目(DepProject) service接口
 *
 * @author lixing
 * @version V1.0 2021-03-24 10:08:12
 */
public interface IDepProjectService {

    /**
     * 创建部门项目
     *
     * @param addDepProjectDTO addDTO
     * @return 部门项目主键
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    String createDepProject(AddDepProjectDTO addDepProjectDTO);

    /**
     * 部门项目详情
     *
     * @param id 主键
     * @return 部门项目do对象
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    DepProject queryDepProjectDetail(String id);

    /**
     * 更新部门项目
     *
     * @param updateDepProjectDTO updateDTO
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    void updateDepProject(UpdateDepProjectDTO updateDepProjectDTO);

    /**
     * 删除部门项目
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    void deleteDepProject(String id);

    /**
     * 查询部门项目
     *
     * @param queryDepProjectDTO queryDTO
     * @return List<DepProject>
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    List<DepProject> queryDepProjectList(QueryDepProjectDTO queryDepProjectDTO);

    /**
     * 分页查询部门项目
     *
     * @param pageQueryDepProjectDTO pageQueryDTO
     * @return IPage<DepProject>
     * @author lixing
     * @version V1.0 2021-03-24 10:08:12
     */
    IPage<DepProject> pageQueryDepProject(PageQueryDepProjectDTO pageQueryDepProjectDTO);

    /**
     * 获取部门关联的项目树
     *
     * @param depIds 部门id列表
     * @return 部门关联的项目树
     * @author lixing
     * @version V1.0 2021/3/22 3:58 下午
     */
    List<ProjectTreeItemVO> queryDepProjectTree(List<String> depIds);

    /**
     * 批量保存部门关联的项目
     *
     * @param addDepProjectDtoList 批量保存入参
     * @author lixing
     * @version V1.0 2021/3/24 10:16 上午
     */
    void saveBatch(List<AddDepProjectDTO> addDepProjectDtoList);

    /**
     * 删除部门关联的项目
     *
     * @param depId 部门id
     * @author lixing
     * @version V1.0 2021/3/24 10:48 上午
     */
    void deleteByDepId(String depId);
}
