package com.persagy.fm.person.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.fm.common.model.vo.DuplicateVO;
import com.persagy.fm.common.model.vo.SimpleObjVO;
import com.persagy.fm.common.response.CommonResult;
import com.persagy.fm.common.response.FmResponseUpsertVO;
import com.persagy.fm.common.response.PageList;
import com.persagy.fm.common.utils.ResultHelper;
import com.persagy.fm.mybatis.annotations.FormatMybatisQueryParams;
import com.persagy.fm.person.model.ConvertPersonTool;
import com.persagy.fm.person.model.Person;
import com.persagy.fm.person.model.dto.*;
import com.persagy.fm.person.model.vo.*;
import com.persagy.fm.person.service.IPersonCheckService;
import com.persagy.fm.person.service.IPersonProjectService;
import com.persagy.fm.person.service.IPersonQueryService;
import com.persagy.fm.person.service.IPersonUpsertService;
import com.persagy.fm.saas.project.model.vo.ProjectTreeItemVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * 人员控制层
 *
 * @author lixing
 * @version V1.0 2021-03-15 17:20:31
 */
@Api(tags = "人员")
@Validated
@RestController
@RequestMapping("persons")
public class PersonController {
    @Autowired
    private IPersonUpsertService personUpsertService;
    @Autowired
    private IPersonQueryService personQueryService;
    @Autowired
    private IPersonCheckService personCheckService;
    @Autowired
    private IPersonProjectService personProjectService;

    @ApiOperation(value = "创建人员")
    @PostMapping("/create")
    public CommonResult<FmResponseUpsertVO> createPerson(
            @Validated @RequestBody AddPerson4FrontEndDTO addPerson4FrontEndDTO) {
        String id = personUpsertService.createPerson(addPerson4FrontEndDTO);
        return ResultHelper.single(new FmResponseUpsertVO(id));
    }

    @ApiOperation(value = "同步创建人员")
    @PostMapping("/sync_create")
    public CommonResult<FmResponseUpsertVO> syncCreatePerson(
            @Validated @RequestBody AddPerson4BackEndDTO addPerson4BackEndDTO) {
        AddPerson4FrontEndDTO addPerson4FrontEndDTO = ConvertPersonTool.INSTANCE.
                convert2AddPerson4FrontEndDTO(addPerson4BackEndDTO);
        String id = personUpsertService.createPerson(addPerson4FrontEndDTO);
        return ResultHelper.single(new FmResponseUpsertVO(id));
    }

    @ApiOperation(value = "人员详情")
    @PostMapping("/get")
    public CommonResult<ResponsePersonItemVO> queryPersonDetail(
            @Valid @RequestBody QueryPersonDetailDTO queryPersonDetailDTO) {
        ResponsePersonItemVO responsePersonItemVO = personQueryService.queryPersonDetail(
                queryPersonDetailDTO.getId(), queryPersonDetailDTO.getDepId());
        return ResultHelper.single(responsePersonItemVO);
    }


    @ApiOperation(value = "更新人员")
    @PostMapping("/update")
    public CommonResult<FmResponseUpsertVO> updatePerson(
            @Valid @RequestBody UpdatePerson4FrontEndDTO updatePerson4FrontEndDTO) {
        personUpsertService.updatePerson(updatePerson4FrontEndDTO);
        return ResultHelper.single(new FmResponseUpsertVO(updatePerson4FrontEndDTO.getId()));
    }


    @ApiOperation(value = "删除人员")
    @PostMapping("/delete")
    public CommonResult deletePerson(
            @RequestBody @Valid DeletePersonDTO deletePersonDTO) {
        personUpsertService.deletePerson(deletePersonDTO.getId());
        return ResultHelper.success("删除成功");
    }

    @ApiOperation(value = "查找人名")
    @PostMapping("/name_quick_search")
    @FormatMybatisQueryParams
    public CommonResult<PageList<SimplePersonListItemVO>> nameQuickSearch(
            @Valid @RequestBody NameQuickSearchDTO nameQuickSearchDTO) {
        List<Person> personList = personQueryService.nameQuickSearch(nameQuickSearchDTO);
        List<SimplePersonListItemVO> resultList = ConvertPersonTool.INSTANCE.convert2SimpleList(personList);
        return ResultHelper.multi(resultList);
    }


    @ApiOperation(value = "人员列表, 分页查询")
    @PostMapping("/query")
    @FormatMybatisQueryParams
    public CommonResult<PageList<ResponsePersonListItemVO>> pageQueryPerson(
            @Valid @RequestBody PageQueryPersonDTO pageQueryPersonDTO) {
        IPage<ResponsePersonListItemVO> pagedList = personQueryService.pageQueryPerson(pageQueryPersonDTO);
        return ResultHelper.multi(pagedList);
    }

    @ApiOperation(value = "根据姓名或用户名分页查询")
    @PostMapping("/queryByNameOrUsername")
    @FormatMybatisQueryParams
    public CommonResult<PageList<ResponsePersonListItemVO>> pageQueryByNameOrUserName(
            @Valid @RequestBody PageQueryPersonByNameOrUserNameDTO pageQueryPersonByNameOrUserNameDTO) {
        IPage<ResponsePersonListItemVO> pagedList = personQueryService.pageQueryByNameOrUserName(pageQueryPersonByNameOrUserNameDTO);
        return ResultHelper.multi(pagedList);
    }

    @ApiOperation(value = "根据用户名分页查询")
    @PostMapping("/queryByUsername")
    @FormatMybatisQueryParams
    public CommonResult<PageList<ResponsePersonListItemVO>> queryByUserName(
            @Valid @RequestBody QueryByUserNameDTO queryByUserNameDTO) {
        IPage<ResponsePersonListItemVO> pagedList = personQueryService.queryByUserName(queryByUserNameDTO);
        return ResultHelper.multi(pagedList);
    }

    @ApiOperation(value = "根据名称分页查询")
    @PostMapping("/queryByName")
    @FormatMybatisQueryParams
    public CommonResult<PageList<ResponsePersonListItemVO>> queryByName(
            @Valid @RequestBody QueryByNameDTO queryByNameDTO) {
        IPage<ResponsePersonListItemVO> pagedList = personQueryService.queryByName(queryByNameDTO);
        return ResultHelper.multi(pagedList);
    }

    @ApiOperation(value = "根据部门id分页查询")
    @PostMapping("/queryByDepId")
    @FormatMybatisQueryParams
    public CommonResult<PageList<ResponsePersonListItemVO>> queryByDepId(
            @Valid @RequestBody PageQueryByDepIdDTO pageQueryByDepIdDTO) {
        IPage<ResponsePersonListItemVO> pagedList = personQueryService.queryByDepId(pageQueryByDepIdDTO);
        return ResultHelper.multi(pagedList);
    }

    @ApiOperation(value = "禁用人员")
    @PostMapping("/disable")
    public CommonResult<FmResponseUpsertVO> disablePerson(
            @Valid @RequestBody PersonIdDTO personIdDTO) {
        personUpsertService.disablePerson(personIdDTO.getId());
        return ResultHelper.single(new FmResponseUpsertVO(personIdDTO.getId()));
    }

    @ApiOperation(value = "解禁人员")
    @PostMapping("/enable")
    public CommonResult<FmResponseUpsertVO> enablePerson(
            @Valid @RequestBody PersonIdDTO personIdDTO) {
        personUpsertService.enablePerson(personIdDTO.getId());
        return ResultHelper.single(new FmResponseUpsertVO(personIdDTO.getId()));
    }

    @ApiOperation(value = "离职人员")
    @PostMapping("/dismiss")
    public CommonResult<FmResponseUpsertVO> dismissPerson(
            @Valid @RequestBody PersonStateChangeDTO personStateChangeDTO) {
        personUpsertService.dismissPerson(personStateChangeDTO.getId());
        return ResultHelper.single(new FmResponseUpsertVO(personStateChangeDTO.getId()));
    }

    @ApiOperation(value = "人员恢复在职")
    @PostMapping("/rejoin")
    public CommonResult<FmResponseUpsertVO> rejoinPerson(
            @Valid @RequestBody PersonStateChangeDTO personStateChangeDTO) {
        personUpsertService.rejoinPerson(personStateChangeDTO.getId());
        return ResultHelper.single(new FmResponseUpsertVO(personStateChangeDTO.getId()));
    }

    @ApiOperation(value = "批量更新")
    @PostMapping("/batch_update")
    public CommonResult batchUpdatePeople(
            @Valid @RequestBody BatchUpdatePersonDTO batchUpdatePersonDTO) {
        personUpsertService.batchUpdatePeople(batchUpdatePersonDTO);
        return ResultHelper.success("批量更新成功");
    }

    @ApiOperation(value = "人员项目树")
    @PostMapping("/projects/tree")
    public CommonResult<PageList<ProjectTreeItemVO>> queryPersonProjectTree(
            @Valid @RequestBody PersonIdDTO personIdDTO) {
        List<ProjectTreeItemVO> projectTreeItemVOList = personProjectService.queryPersonProjectTree(personIdDTO.getId());
        return ResultHelper.multi(projectTreeItemVOList);
    }

    @ApiOperation(value = "获取最新工号")
    @PostMapping("/latestJobNumber")
    public CommonResult<LatestJobNumberVO> queryLatestJobNumber() {
        LatestJobNumberVO latestJobNumberVO = personQueryService.queryLatestJobNumber();
        return ResultHelper.single(latestJobNumberVO);
    }

    @ApiOperation(value = "校验人员属性重复")
    @PostMapping("/checkDuplicate")
    public CommonResult<DuplicateVO> checkDuplicate(@RequestBody CheckDuplicateDTO checkDuplicateDTO) {
        DuplicateVO duplicateVO = new DuplicateVO();
        duplicateVO.setDuplicate(false);

        Person person = null;
        if (StringUtils.isNotBlank(checkDuplicateDTO.getPersonId())) {
            person = personQueryService.queryPersonProfile(checkDuplicateDTO.getPersonId());
        }
        // 校验身份证号
        if (personCheckService.checkDuplicateIdNumber(checkDuplicateDTO.getIdNumber(), person)) {
            duplicateVO.setDuplicate(true);
            return ResultHelper.single(duplicateVO, "身份证号重复");
        }
        // 校验账号
        if (personCheckService.checkDuplicateUserName(checkDuplicateDTO.getUsername(), person)) {
            duplicateVO.setDuplicate(true);
            return ResultHelper.single(duplicateVO, "账号已被使用");
        }
        // 校验手机号
        if (personCheckService.checkDuplicatePhoneNum(checkDuplicateDTO.getPhoneNumber(), person)) {
            duplicateVO.setDuplicate(true);
            return ResultHelper.single(duplicateVO, "手机号已被使用");
        }
        // 校验工号
        if (personCheckService.checkDuplicateJobNumber(checkDuplicateDTO.getJobNumber(), person)) {
            duplicateVO.setDuplicate(true);
            return ResultHelper.single(duplicateVO, "工号已被使用");
        }
        // 校验邮箱
        if (personCheckService.checkDuplicateMail(checkDuplicateDTO.getMail(), person)) {
            duplicateVO.setDuplicate(true);
            return ResultHelper.single(duplicateVO, "邮箱已被使用");
        }
        return ResultHelper.single(duplicateVO);
    }

    @ApiOperation(value = "根据账号id查询人员信息")
    @PostMapping("/queryByAccountId")
    public CommonResult<ResponsePersonItemVO> queryByAccountId(
            @Valid @RequestBody QueryByAccountIdDTO queryByAccountIdDTO) {
        ResponsePersonItemVO vo = personQueryService.queryDetailByAccountId(
                queryByAccountIdDTO.getAccountId(), queryByAccountIdDTO.getDepId());
        return ResultHelper.single(vo);
    }

    @ApiOperation(value = "根据主键列表查询人员信息")
    @PostMapping("/queryByIds")
    public CommonResult<PageList<ResponsePersonListItemVO>> queryByIds(
            @Valid @RequestBody QueryByIdsDTO queryByIdsDTO) {
        IPage<ResponsePersonListItemVO> vos = personQueryService.queryDetailByIds(queryByIdsDTO.getIds());
        return ResultHelper.multi(vos);
    }

    @ApiOperation(value = "查询人员的账号id")
    @PostMapping("/queryAccountId")
    public CommonResult<AccountIdVO> queryAccountId(
            @Valid @RequestBody PersonIdDTO personIdDTO) {
        String accountId = personQueryService.queryAccountId(personIdDTO.getId());
        AccountIdVO accountIdVO = new AccountIdVO();
        accountIdVO.setAccountId(accountId);
        return ResultHelper.single(accountIdVO);
    }

    @ApiOperation(value = "查询所有员工的姓名")
    @PostMapping("/queryAllPersonName")
    public CommonResult<PageList<SimpleObjVO>> queryAllPersonName() {
        List<SimpleObjVO> personNameList = personQueryService.queryAllPersonName();
        IPage<SimpleObjVO> result = new Page<>();
        result.setRecords(personNameList);
        if (CollectionUtils.isEmpty(personNameList)) {
            result.setTotal(0);
        } else {
            result.setTotal(personNameList.size());
        }

        return ResultHelper.multi(result);
    }

    @ApiOperation(value = "根据岗位/角色主键查询员工信息")
    @PostMapping("/queryByRoleIds")
    public CommonResult<PageList<ResponsePersonListItemVO>> pageQueryByRoleIds(
            @Valid @RequestBody PageQueryByRoleIdsDTO pageQueryByRoleIdsDTO) {
        IPage<ResponsePersonListItemVO> pagedList = personQueryService.pageQueryByRoleIds(pageQueryByRoleIdsDTO);
        return ResultHelper.multi(pagedList);
    }
}

