package com.persagy.fm.person.controller;

import com.persagy.fm.common.model.dto.EnumQueryDTO;
import com.persagy.fm.common.model.vo.EnumVO;
import com.persagy.fm.common.response.CommonResult;
import com.persagy.fm.common.response.PageList;
import com.persagy.fm.common.utils.ResultHelper;
import com.persagy.fm.person.service.IPersonEnumService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * @author lixing
 * @version V1.0 2021/3/12 6:36 下午
 **/
@RestController
@Validated
@RequestMapping("/persons/enums")
@Api(tags = "人员枚举")
public class PersonEnumController {
    @Autowired
    IPersonEnumService personEnumService;

    @PostMapping("/get")
    @ApiOperation(value="查询(certificate_type:证件类型, person_type:人员类型)")
    public CommonResult<PageList<EnumVO>> queryEnum(
            @Valid @RequestBody EnumQueryDTO enumQueryDTO) {
        String type = enumQueryDTO.getType();
        List<EnumVO> resultList;
        switch (type){
            case "certificate_type":
                resultList = personEnumService.queryCertificateTypeEnum();
                break;
            case "person_type":
                resultList = personEnumService.queryPersonTypeEnum();
                break;
            default:
                throw new IllegalArgumentException("暂不提供这种类型的枚举信息查询");
        }
        return ResultHelper.multi(resultList);
    }
}
