package com.persagy.fm.person.service;

import com.persagy.fm.person.model.Person;

import java.util.List;

/**
 * 人员(Person) 校验接口
 *
 * @author lixing
 * @version V1.0 2021-03-15 17:20:31
 */
public interface IPersonCheckService {
    /**
     * 校验直接上级
     *
     * @param leaderId 直接上级id
     * @param person   人员对象
     * @author lixing
     * @version V1.0 2021/4/22 11:43 上午
     */
    void checkLeader(String leaderId, Person person);

    /**
     * 批量更新时，验证人员的主副岗是否重复
     *
     * @param mainDuty    更新条件中的主岗，为空代表不更新
     * @param otherDuties 更新条件中的副岗，为空代表不更新
     * @param personId    人员id
     * @author lixing
     * @version V1.0 2021/4/21 7:44 下午
     */
    void checkMainAndOtherDutiesWhenBatchUpdate(String mainDuty, List<String> otherDuties, String personId);
    /**
     * 校验重复的手机号
     *
     * @param phoneNumber 手机号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:39 上午
     */
    Boolean checkDuplicatePhoneNum(String phoneNumber);

    /**
     * 校验重复的手机号，如果没变动不校验
     *
     * @param phoneNumber 手机号
     * @param person      当前人员
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:39 上午
     */
    Boolean checkDuplicatePhoneNum(String phoneNumber, Person person);

    /**
     * 校验重复的身份证号
     *
     * @param idNumber 身份证号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:40 上午
     */
    Boolean checkDuplicateIdNumber(String idNumber);

    /**
     * 校验重复的身份证号，如果没变动不校验
     *
     * @param idNumber 身份证号
     * @param person   当前人员
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:40 上午
     */
    Boolean checkDuplicateIdNumber(String idNumber, Person person);

    /**
     * 校验重复的登录名
     *
     * @param username 登录名
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:41 上午
     */
    Boolean checkDuplicateUserName(String username);

    /**
     * 校验重复的登录名，如果没变动不校验
     *
     * @param username 登录名
     * @param person   当前人员
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:40 上午
     */
    Boolean checkDuplicateUserName(String username, Person person);

    /**
     * 校验重复的工号
     *
     * @param jobNumber 工号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:41 上午
     */
    Boolean checkDuplicateJobNumber(String jobNumber);

    /**
     * 校验重复的工号，如果没变动不校验
     *
     * @param jobNumber 工号
     * @param person    当前人员
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:40 上午
     */
    Boolean checkDuplicateJobNumber(String jobNumber, Person person);

    /**
     * 校验重复的邮箱
     *
     * @param mail 邮箱
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:41 上午
     */
    Boolean checkDuplicateMail(String mail);

    /**
     * 校验重复的邮箱，如果没变动不校验
     *
     * @param mail   邮箱
     * @param person 当前人员
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/7 11:40 上午
     */
    Boolean checkDuplicateMail(String mail, Person person);
}
