package com.persagy.fm.person.service.impl;

import com.persagy.fm.person.model.Person;
import com.persagy.fm.person.service.IPersonProjectService;
import com.persagy.fm.person.service.IPersonQueryService;
import com.persagy.fm.saas.accountproject.model.SaasAccountProject;
import com.persagy.fm.saas.accountproject.model.dto.QuerySaasAccountProjectDTO;
import com.persagy.fm.saas.accountproject.service.ISaasAccountProjectService;
import com.persagy.fm.saas.project.model.vo.ProjectTreeItemVO;
import com.persagy.fm.saas.project.service.ISaasProjectService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 人员关联的项目service层
 *
 * @author lixing
 * @version V1.0 2021/3/22 11:54 上午
 **/
@Service
public class PersonProjectServiceImpl implements IPersonProjectService {
    @Autowired
    ISaasProjectService saasProjectService;
    @Autowired
    IPersonQueryService personQueryService;
    @Autowired
    ISaasAccountProjectService accountProjectService;


    @Override
    public List<ProjectTreeItemVO> queryPersonProjectTree(String personId) {
        // 根据人员id查询人员关联的项目，然后组装项目树
        Person person = personQueryService.queryPersonProfile(personId);
        String accountId = person.getAccountId();
        QuerySaasAccountProjectDTO querySaasAccountProjectDTO = new QuerySaasAccountProjectDTO();
        querySaasAccountProjectDTO.setAccountId(accountId);
        List<SaasAccountProject> saasAccountProjects = accountProjectService.querySaasAccountProjectList(querySaasAccountProjectDTO);
        if (CollectionUtils.isEmpty(saasAccountProjects)) {
            return Lists.newArrayList();
        }
        List<String> projectIds = saasAccountProjects.stream()
                .map(SaasAccountProject::getProjectId)
                .collect(Collectors.toList());
        return saasProjectService.queryProjectTree(projectIds);
    }
}
