package com.persagy.fm.saas.account.service.impl;

import com.persagy.fm.common.constant.enums.ValidEnum;
import com.persagy.fm.common.response.FmResponseContent;
import com.persagy.fm.saas.account.client.SaasAccountClient;
import com.persagy.fm.saas.account.constant.enums.AccountTypeEnum;
import com.persagy.fm.saas.account.model.dto.*;
import com.persagy.fm.saas.account.model.vo.SaasAccountListItemVO;
import com.persagy.fm.saas.account.service.ISaasAccountService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 运维中心账号service层
 *
 * @author lixing
 * @version V1.0 2021/4/6 2:33 下午
 **/
@Service
public class SaasAccountServiceImpl implements ISaasAccountService {
    @Autowired
    SaasAccountClient saasAccountClient;

    @Override
    public String createSaasAccount(AddSaasAccountDTO addSaasAccountDTO) {
        addSaasAccountDTO.setDefaultValueNoAccountId();
        addSaasAccountDTO.setAccountType(AccountTypeEnum.COMMON.getType());
        addSaasAccountDTO.setValidStartTime(new Date());
        FmResponseContent<String> saasAccount = saasAccountClient.
                createSaasAccount(addSaasAccountDTO);
        return saasAccount.getContent();
    }

    @Override
    public List<SaasAccountListItemVO> querySaasAccountList(PageQuerySaasAccountDTO pageQuerySaasAccountDTO) {
        pageQuerySaasAccountDTO.setPageInfo();
        FmResponseContent<List<SaasAccountListItemVO>> listFmResponseContent = saasAccountClient.
                querySaasAccountPageList(pageQuerySaasAccountDTO);
        return listFmResponseContent.getContent();
    }

    @Override
    public void disableAccount(DeleteSaasAccountDTO deleteSaasAccountDTO) {
        deleteSaasAccountDTO.setDefaultValueNoAccountId();
        saasAccountClient.disableAccount(deleteSaasAccountDTO);
    }

    @Override
    public void enableAccount(EnableSaasAccountDTO enableSaasAccountDTO) {
        // 判断账号的状态，如果账号是禁用状态重新启用
        String accountId = enableSaasAccountDTO.getAccountId();
        SaasAccountListItemVO saasAccountListItemVO = querySaasAccountDetail(accountId);
        if (saasAccountListItemVO != null &&
                ValidEnum.FALSE.getType().equals(saasAccountListItemVO.getValid())) {
            enableSaasAccountDTO.setDefaultValueNoAccountId();
            saasAccountClient.enableAccount(enableSaasAccountDTO);
        }
    }

    @Override
    public void updateSaasAccount(UpdateSaasAccountDTO updateSaasAccountDTO) {
        updateSaasAccountDTO.setDefaultValueNoAccountId();
        saasAccountClient.updateSaasAccount(updateSaasAccountDTO);
    }

    @Override
    public SaasAccountListItemVO querySaasAccountDetail(String accountId) {
        PageQuerySaasAccountDTO pageQuerySaasAccountDTO = new PageQuerySaasAccountDTO();
        pageQuerySaasAccountDTO.setAccountId(accountId);
        List<SaasAccountListItemVO> accountListItemVOList = querySaasAccountList(pageQuerySaasAccountDTO);
        if (!CollectionUtils.isEmpty(accountListItemVOList)) {
            return accountListItemVOList.get(0);
        } else {
            throw new IllegalArgumentException("未获取到账号信息，accountId:" + accountId);
        }
    }

    @Override
    public List<SaasAccountListItemVO> querySaasAccountListByAccountIds(List<String> accountIds) {
        if (CollectionUtils.isEmpty(accountIds)) {
            return Lists.newArrayList();
        }
        PageQuerySaasAccountDTO pageQuerySaasAccountDTO = new PageQuerySaasAccountDTO();
        pageQuerySaasAccountDTO.setAccountIds(accountIds);
        pageQuerySaasAccountDTO.setSize(accountIds.size());
        pageQuerySaasAccountDTO.setPageInfo();
        FmResponseContent<List<SaasAccountListItemVO>> listFmResponseContent = saasAccountClient.
                querySaasAccountPageList(pageQuerySaasAccountDTO);
        return listFmResponseContent.getContent();
    }
}
