package com.persagy.fm.saas.accountproject.client;

import com.persagy.fm.common.client.FeignConfig;
import com.persagy.fm.common.response.FmResponseContent;
import com.persagy.fm.common.response.FmResponseMsg;
import com.persagy.fm.saas.account.model.dto.AddSaasAccountDTO;
import com.persagy.fm.saas.account.model.dto.PageQuerySaasAccountDTO;
import com.persagy.fm.saas.account.model.dto.UpdateSaasAccountDTO;
import com.persagy.fm.saas.account.model.vo.SaasAccountListItemVO;
import com.persagy.fm.saas.accountproject.client.fallback.SaasAccountProjectFallbackFactory;
import com.persagy.fm.saas.accountproject.model.SaasAccountProject;
import com.persagy.fm.saas.accountproject.model.dto.AddSaasAccountProjectDTO;
import com.persagy.fm.saas.accountproject.model.dto.DeleteSaasAccountProjectDTO;
import com.persagy.fm.saas.accountproject.model.dto.QuerySaasAccountProjectDTO;
import com.persagy.fm.saas.accountproject.model.dto.ResetSaasAccountProjectDTO;
import com.persagy.fm.saas.project.constant.FeignClientNameConstants;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * 运维平台账号项目接口feign客户端
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:13 下午
 **/
@FeignClient(name = FeignClientNameConstants.SAAS, fallbackFactory = SaasAccountProjectFallbackFactory.class,
        configuration = FeignConfig.class)
public interface SaasAccountProjectClient {
    /**
     * 查询账号项目列表
     *
     * @param querySaasAccountProjectDTO 查询条件
     * @return 账号项目列表
     * @author lixing
     * @version V1.0 2021/3/23 11:57 上午
     */
    @PostMapping("/accountProject/querySaasAccountProjectList")
    FmResponseContent<List<SaasAccountProject>> querySaasAccountProjectList(
            @RequestBody QuerySaasAccountProjectDTO querySaasAccountProjectDTO);

    /**
     * 创建账号项目
     *
     * @param addSaasAccountProjectDTO 创建账号项目入参
     * @return 账号项目id
     * @author lixing
     * @version V1.0 2021/3/23 11:57 上午
     */
    @PostMapping("/accountProject/createSaasAccountProject")
    FmResponseContent<String> createSaasAccountProject(@RequestBody AddSaasAccountProjectDTO addSaasAccountProjectDTO);

    /**
     * 删除账号项目
     *
     * @param deleteSaasAccountProjectDTO 删除账号项目入参
     * @author lixing
     * @return FmResponseMsg
     * @version V1.0 2021/3/23 11:57 上午
     */
    @PostMapping("/accountProject/deleteSaasAccountProject")
    FmResponseMsg deleteSaasAccountProject(@RequestBody DeleteSaasAccountProjectDTO deleteSaasAccountProjectDTO);

    /**
     * 重置账号和项目的关联关系
     *
     * @param resetSaasAccountProjectDTO  重置账号项目入参
     * @return FmResponseMsg
     * @author lixing
     * @version V1.0 2021/4/8 2:44 下午
     */
    @PostMapping("/accountProject/resetSaasAccountProject")
    FmResponseMsg resetSaasAccountProject(@RequestBody ResetSaasAccountProjectDTO resetSaasAccountProjectDTO);
}
