package com.persagy.fm.saas.project.client;

import com.persagy.fm.common.client.FeignConfig;
import com.persagy.fm.common.response.FmResponseContent;
import com.persagy.fm.common.response.FmResponseItem;
import com.persagy.fm.saas.project.client.fallback.SaasProjectClientFallbackFactory;
import com.persagy.fm.saas.project.constant.FeignClientNameConstants;
import com.persagy.fm.saas.project.model.SaasArea;
import com.persagy.fm.saas.project.model.SaasProject;
import com.persagy.fm.saas.project.model.dto.QueryAllowSaasAreaProjectTreeDTO;
import com.persagy.fm.saas.project.model.dto.QuerySaasAreaDTO;
import com.persagy.fm.saas.project.model.dto.QuerySaasAreaProjectTreeDTO;
import com.persagy.fm.saas.project.model.dto.QuerySaasProjectDTO;
import com.persagy.fm.saas.project.model.vo.SaasAreaProjectTreeVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * 接口说明
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:13 下午
 **/
@FeignClient(name = FeignClientNameConstants.SAAS, fallbackFactory = SaasProjectClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface SaasProjectClient {
    /**
     * 查询运维系统区域列表
     *
     * @param querySaasAreaDTO 查询运维系统区域列表入参
     * @return 区域列表
     * @author lixing
     * @version V1.0 2021/3/26 5:47 下午
     */
    @PostMapping("/area/querySaasAreaList")
    FmResponseContent<List<SaasArea>> querySaasAreaList(@RequestBody QuerySaasAreaDTO querySaasAreaDTO);

    /**
     * 查询区域项目树（无权限控制），不支持
     *
     * @param querySaasAreaProjectTreeDTO 查询区域项目树入参
     * @return 区域项目树
     * @author lixing
     * @version V1.0 2021/3/26 5:46 下午
     */
    @PostMapping("/project/queryAreaProjectTree")
    FmResponseContent<List<SaasAreaProjectTreeVO>> queryAreaProjectTree(
             @RequestBody QuerySaasAreaProjectTreeDTO querySaasAreaProjectTreeDTO);

    /**
     * 查询区域项目树（有权限控制）
     *
     * @param queryAllowSaasAreaProjectTreeDTO 查询区域项目树入参
     * @return 区域项目树
     * @author lixing
     * @version V1.0 2021/3/29 6:17 下午
     */
    @PostMapping("/project/queryAllowAreaProjectTree")
    FmResponseContent<List<SaasAreaProjectTreeVO>> queryAllowAreaProjectTree(
            @RequestBody QueryAllowSaasAreaProjectTreeDTO queryAllowSaasAreaProjectTreeDTO);

    /**
     * 查询项目列表
     *
     * @param querySaasProjectDTO 查询项目入参
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/3/29 3:06 下午
     */
    @PostMapping("/project/querySaasProjectList")
    FmResponseContent<List<SaasProject>> queryProjectList(QuerySaasProjectDTO querySaasProjectDTO);
}
