package com.persagy.fm.saas.role.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Maps;
import com.persagy.fm.common.context.AppContext;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.common.response.FmResponseContent;
import com.persagy.fm.saas.account.constant.enums.AccountBelongEnum;
import com.persagy.fm.saas.accountrole.service.ISaasAccountRoleService;
import com.persagy.fm.saas.role.client.SaasRoleClient;
import com.persagy.fm.saas.role.constant.enums.RoleTypeEnum;
import com.persagy.fm.saas.role.model.ConvertSaasRoleTool;
import com.persagy.fm.saas.role.model.SaasRole;
import com.persagy.fm.saas.role.model.dto.*;
import com.persagy.fm.saas.role.service.ISaasRoleService;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * (SaasRole) service层
 *
 * @author lixing
 * @version V1.0 2021-03-22 19:04:17
 */
@Service
public class SaasRoleServiceImpl implements ISaasRoleService {
    @Autowired
    SaasRoleClient saasRoleClient;
    @Autowired
    ISaasAccountRoleService saasAccountRoleService;

    /**
     * 创建
     *
     * @return String 主键
     * @author lixing
     * @version V1.0 2021-03-22 19:04:17
     */
    @Override
    public String createSaasRole(AddSaasRoleDTO addSaasRoleDTO) {
        SaasRole saasRole = ConvertSaasRoleTool.INSTANCE.convertAddDto2Entity(addSaasRoleDTO);
        // 设置默认值
        setDefaultValue(saasRole);
        FmResponseContent<String> response = saasRoleClient.createSaasRole(saasRole);
        return response.getContent();
    }

    /**
     * 校验角色名称是否重复
     *
     * @param roleName 角色名称
     * @author lixing
     * @version V1.0 2021/4/20 6:40 下午
     */
    private void checkDuplicateName(String roleName) {
        QuerySaasRoleByNameDTO querySaasRoleByNameDTO = new QuerySaasRoleByNameDTO();
        querySaasRoleByNameDTO.setDefaultValue();
        querySaasRoleByNameDTO.setRoleName(roleName);
        FmResponseContent<List<SaasRole>> responseContent = saasRoleClient.querySaasRoleByName(querySaasRoleByNameDTO);
        if (responseContent.getCount() > 0) {
            throw new IllegalArgumentException("该名称已被使用");
        }
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param saasRole 实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(SaasRole saasRole) {
        saasRole.setAccountId(AppContext.getContext().getAccountId());
        saasRole.setAccountBelong(AccountBelongEnum.BUSINESS.getType());
        saasRole.setAppId(DefaultAppContext.getContext().getAppId());
        saasRole.setGroupCode(DefaultAppContext.getContext().getGroupCode());
        saasRole.setRoleCode(String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 更新
     *
     * @author lixing
     * @version V1.0 2021-03-22 19:04:17
     */
    @Override
    public void updateSaasRole(UpdateSaasRoleDTO updateSaasRoleDTO) {
        // 查询要更新的对象
        SaasRole saasRole = getSaasRole(updateSaasRoleDTO.getId());

        if (saasRole == null) {
            throw new IllegalArgumentException("找不到要更新的数据，id: " + updateSaasRoleDTO.getId());
        }

//        if (StringUtils.isNotBlank(updateSaasRoleDTO.getRoleName()) &&
//                !saasRole.getRoleName().equals(updateSaasRoleDTO.getRoleName())) {
//            checkDuplicateName(updateSaasRoleDTO.getRoleName());
//        }
        // 将更新的属性赋值给查询到的对象
        saasRole = ConvertSaasRoleTool.INSTANCE.convertUpdateDto2Entity(saasRole, updateSaasRoleDTO);
        saasRole.setDefaultValue();
        saasRoleClient.updateSaasRole(saasRole);
    }

    /**
     * 获取运维平台角色对象
     *
     * @param roleId 角色id
     * @return 角色对象
     * @author lixing
     * @version V1.0 2021/3/24 3:47 下午
     */
    private SaasRole getSaasRole(String roleId) {
        PageQuerySaasRoleDTO pageQuerySaasRoleDTO = new PageQuerySaasRoleDTO();
        pageQuerySaasRoleDTO.setDefaultValue();
        pageQuerySaasRoleDTO.setId(roleId);
        FmResponseContent<List<SaasRole>> listFmResponseContent = saasRoleClient.queryRolePageList(pageQuerySaasRoleDTO);
        if (listFmResponseContent.getCount() <= 0) {
            return null;
        }
        return listFmResponseContent.getContent().get(0);
    }

    /**
     * 校验是否可删除
     *
     * @param id 主键
     * @return do类
     * @author lixing
     * @version V1.0 2021-03-22 19:04:17
     */
    public SaasRole checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除SaasRole时发生异常，主键为空");
        }

        SaasRole saasRole = getSaasRole(id);

        if (saasRole == null) {
            throw new IllegalArgumentException("删除SaasRole时发生异常，找不到要删除的数据，id:" + id);
        }

        if (RoleTypeEnum.MANAGER.getType().equals(saasRole.getRoleType())) {
            throw new IllegalArgumentException("集团管理员角色不可删除");
        }

        return saasRole;
    }

    /**
     * 删除
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-03-22 19:04:17
     */
    @Override
    public void deleteSaasRole(String id) {
        // 校验是否可删除
        checkDeletable(id);
        // 删除角色
        DeleteSaasRoleDTO deleteSaasRoleDTO = new DeleteSaasRoleDTO();
        deleteSaasRoleDTO.setDefaultValue();
        deleteSaasRoleDTO.setId(id);
        saasRoleClient.deleteSaasRole(deleteSaasRoleDTO);
    }

    /**
     * 分页查询
     *
     * @return IPage<SaasRole>
     * @author lixing
     * @version V1.0 2021-03-22 19:04:17
     */
    @Override
    public IPage<SaasRole> pageQuerySaasRole(PageQuerySaasRoleDTO pageQuerySaasRoleDTO) {
        // 调用运维平台分页查询接口
        pageQuerySaasRoleDTO.setDefaultValue();
        pageQuerySaasRoleDTO.setCountFunction(true);
        pageQuerySaasRoleDTO.setCountMenu(true);
        FmResponseContent<List<SaasRole>> saasResponse = saasRoleClient.queryRolePageList(pageQuerySaasRoleDTO);
        Integer count = saasResponse.getCount();
        List<SaasRole> content = saasResponse.getContent();

        // 将返回结果封装为IPage对象
        IPage<SaasRole> result = new Page<>();
        result.setRecords(content);
        result.setTotal(Long.parseLong(count.toString()));
        return result;
    }

    @Override
    public List<SaasRole> querySaasRoleList(QuerySaasRoleDTO querySaasRoleDTO) {
        querySaasRoleDTO.setDefaultValue();
        FmResponseContent<List<SaasRole>> responseContent = saasRoleClient.queryRoleList(querySaasRoleDTO);
        return responseContent.getContent();
    }

    @Override
    public List<SaasRole> querySaasRoleListByRoleIdList(List<String> roleIdList) {
        QuerySaasRoleDTO querySaasRoleDTO = new QuerySaasRoleDTO();
        querySaasRoleDTO.setRoleIds(roleIdList);
        return querySaasRoleList(querySaasRoleDTO);
    }

    @Override
    public Map<String, SaasRole> querySaasRoleMapByRoleIdList(List<String> roleIdList) {
        List<SaasRole> saasRoles = querySaasRoleListByRoleIdList(roleIdList);
        if (CollectionUtils.isEmpty(saasRoles)) {
            return Maps.newHashMap();
        }
        return saasRoles.stream().collect(
                Collectors.toMap(
                        SaasRole::getId,
                        saasRole -> saasRole,
                        (k1, k2) -> k1
                )
        );
    }

    @Override
    public List<SaasRole> queryByName(QuerySaasRoleByNameDTO querySaasRoleByNameDTO) {
        FmResponseContent<List<SaasRole>> responseContent = saasRoleClient.querySaasRoleByName(querySaasRoleByNameDTO);
        return responseContent.getContent();
    }

    @Override
    public SaasRole querySaasRoleDetail(String id) {
        if (StringUtils.isBlank(id)) {
            return null;
        }
        List<SaasRole> saasRoles = querySaasRoleListByRoleIdList(Lists.newArrayList(id));
        if (CollectionUtils.isEmpty(saasRoles)) {
            return null;
        } else {
            return saasRoles.get(0);
        }
    }
}
