package com.persagy.fm.workflow.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.IoUtil;
import com.persagy.fm.common.response.CommonResult;
import com.persagy.fm.common.response.PageList;
import com.persagy.fm.common.utils.ResultHelper;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.engine.*;
import org.flowable.engine.runtime.Execution;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.image.ProcessDiagramGenerator;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 工作流控制器
 * @author Charlie Yu
 * @date 2021-04-26
 */
@RestController
@RequestMapping("workflow")
public class WorkflowController {

    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private ProcessEngine processEngine;

    /**
     * 启动流程实例
     * @return
     */
    @RequestMapping(value = "start")
    public String startInstance() {
        Map<String, Object> valueMap = new HashMap<>(16);
        valueMap.put("userId", "xxxxxxxx");
        valueMap.put("userName", "yyyyyyyyy");
        valueMap.put("taskUser", "zzzzzzz");
        valueMap.put("money", 1000);
        ProcessInstance instance = runtimeService.startProcessInstanceByKey("Expense", valueMap);
        return instance.getId();
    }

    /**
     * 查询待办
     * @param userId
     * @return
     */
    @RequestMapping(value = "queryTodo")
    public CommonResult<PageList<String>> queryTodo(@RequestBody String userId) {
        List<Task> tasks = taskService.createTaskQuery().taskAssignee(userId).orderByTaskCreateTime().desc().list();
        List<String> ids = CollUtil.getFieldValues(tasks, "id", String.class);
        return ResultHelper.multi(ids);
    }

    /**
     * 审批通过
     * @param taskId
     * @return
     */
    @RequestMapping(value = "apply")
    public String apply(@RequestBody String taskId) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        if (task == null) {
            throw new RuntimeException("流程不存在");
        }
        //通过审核
        HashMap<String, Object> map = new HashMap<>(16);
        map.put("outcome", "通过");
        taskService.complete(taskId, map);
        return "processed ok!";
    }

    /**
     * 审批不通过
     * @param taskId
     * @return
     */
    @RequestMapping(value = "reject")
    public String reject(@RequestBody String taskId) {
        HashMap<String, Object> map = new HashMap<>(16);
        map.put("outcome", "驳回");
        taskService.complete(taskId, map);
        return "reject";
    }

    /**
     * 生成实例流程图
     * @param httpServletResponse
     *
     * @param processId
     * @throws Exception
     */
    @RequestMapping(value = "processDiagram")
    public void genProcessDiagram(HttpServletResponse httpServletResponse, @RequestBody String processId) throws Exception {
        ProcessInstance pi = runtimeService.createProcessInstanceQuery().processInstanceId(processId).singleResult();

        //流程走完的不显示图
        if (pi == null) {
            return;
        }
        Task task = taskService.createTaskQuery().processInstanceId(pi.getId()).singleResult();
        //使用流程实例ID，查询正在执行的执行对象表，返回流程实例对象
        String instanceId = task.getProcessInstanceId();
        List<Execution> executions = runtimeService
                .createExecutionQuery()
                .processInstanceId(instanceId)
                .list();

        //得到正在执行的Activity的Id
        List<String> activityIds = new ArrayList<>();
        List<String> flows = new ArrayList<>();
        for (Execution exe : executions) {
            List<String> ids = runtimeService.getActiveActivityIds(exe.getId());
            activityIds.addAll(ids);
        }

        //获取流程图
        BpmnModel bpmnModel = repositoryService.getBpmnModel(pi.getProcessDefinitionId());
        ProcessEngineConfiguration configuration = processEngine.getProcessEngineConfiguration();
        ProcessDiagramGenerator diagramGenerator = configuration.getProcessDiagramGenerator();
        try(InputStream in = diagramGenerator.generateDiagram(bpmnModel,
                "png", activityIds, flows, "宋体","宋体", "宋体",
                configuration.getClassLoader(), 1.0, true); OutputStream out = httpServletResponse.getOutputStream()){
            IoUtil.copy(in, out);
        }
    }
}
